==> sqlx-core-0.5.8.crate: sqlx-core-0.5.8/src/sqlite/statement/mod.rs <==
use crate::column::ColumnIndex;
use crate::error::Error;
use crate::ext::ustr::UStr;
use crate::sqlite::{Sqlite, SqliteArguments, SqliteColumn, SqliteTypeInfo};
use crate::statement::Statement;
use crate::HashMap;
use either::Either;
use std::borrow::Cow;
use std::sync::Arc;

mod handle;
mod r#virtual;
mod worker;

pub(crate) use handle::{StatementHandle, StatementHandleRef};
pub(crate) use r#virtual::VirtualStatement;
pub(crate) use worker::StatementWorker;

#[derive(Debug, Clone)]
#[allow(clippy::rc_buffer)]
pub struct SqliteStatement<'q> {
    pub(crate) sql: Cow<'q, str>,
    pub(crate) parameters: usize,
    pub(crate) columns: Arc<Vec<SqliteColumn>>,
    pub(crate) column_names: Arc<HashMap<UStr, usize>>,
}

impl<'q> Statement<'q> for SqliteStatement<'q> {
    type Database = Sqlite;

    fn to_owned(&self) -> SqliteStatement<'static> {
        SqliteStatement::<'static> {
            sql: Cow::Owned(self.sql.clone().into_owned()),
            parameters: self.parameters,
            columns: Arc::clone(&self.columns),
            column_names: Arc::clone(&self.column_names),
        }
    }

    fn sql(&self) -> &str {
        &self.sql
    }

    fn parameters(&self) -> Option<Either<&[SqliteTypeInfo], usize>> {
        Some(Either::Right(self.parameters))
    }

    fn columns(&self) -> &[SqliteColumn] {
        &self.columns
    }

    impl_statement_query!(SqliteArguments<'_>);
}

impl ColumnIndex<SqliteStatement<'_>> for &'_ str {
    fn index(&self, statement: &SqliteStatement<'_>) -> Result<usize, Error> {
        statement
            .column_names
            .get(*self)
            .ok_or_else(|| Error::ColumnNotFound((*self).into()))
            .map(|v| *v)
    }
}

#[cfg(feature = "any")]
impl<'q> From<SqliteStatement<'q>> for crate::any::AnyStatement<'q> {
    #[inline]
    fn from(statement: SqliteStatement<'q>) -> Self {
        crate::any::AnyStatement::<'q> {
            columns: statement
                .columns
                .iter()
                .map(|col| col.clone().into())
                .collect(),
            column_names: statement.column_names,
            parameters: Some(Either::Right(statement.parameters)),
            sql: statement.sql,
        }
    }
}

==> sqlx-core-0.5.11.crate: sqlx-core-0.5.11/src/sqlite/statement/mod.rs <==
use crate::column::ColumnIndex;
use crate::error::Error;
use crate::ext::ustr::UStr;
use crate::sqlite::{Sqlite, SqliteArguments, SqliteColumn, SqliteTypeInfo};
use crate::statement::Statement;
use crate::HashMap;
use either::Either;
use std::borrow::Cow;
use std::sync::Arc;

mod handle;
mod r#virtual;

pub(crate) use handle::StatementHandle;
pub(crate) use r#virtual::VirtualStatement;

#[derive(Debug, Clone)]
#[allow(clippy::rc_buffer)]
pub struct SqliteStatement<'q> {
    pub(crate) sql: Cow<'q, str>,
    pub(crate) parameters: usize,
    pub(crate) columns: Arc<Vec<SqliteColumn>>,
    pub(crate) column_names: Arc<HashMap<UStr, usize>>,
}

impl<'q> Statement<'q> for SqliteStatement<'q> {
    type Database = Sqlite;

    fn to_owned(&self) -> SqliteStatement<'static> {
        SqliteStatement::<'static> {
            sql: Cow::Owned(self.sql.clone().into_owned()),
            parameters: self.parameters,
            columns: Arc::clone(&self.columns),
            column_names: Arc::clone(&self.column_names),
        }
    }

    fn sql(&self) -> &str {
        &self.sql
    }

    fn parameters(&self) -> Option<Either<&[SqliteTypeInfo], usize>> {
        Some(Either::Right(self.parameters))
    }

    fn columns(&self) -> &[SqliteColumn] {
        &self.columns
    }

    impl_statement_query!(SqliteArguments<'_>);
}

impl ColumnIndex<SqliteStatement<'_>> for &'_ str {
    fn index(&self, statement: &SqliteStatement<'_>) -> Result<usize, Error> {
        statement
            .column_names
            .get(*self)
            .ok_or_else(|| Error::ColumnNotFound((*self).into()))
            .map(|v| *v)
    }
}

#[cfg(feature = "any")]
impl<'q> From<SqliteStatement<'q>> for crate::any::AnyStatement<'q> {
    #[inline]
    fn from(statement: SqliteStatement<'q>) -> Self {
        crate::any::AnyStatement::<'q> {
            columns: statement
                .columns
                .iter()
                .map(|col| col.clone().into())
                .collect(),
            column_names: statement.column_names,
            parameters: Some(Either::Right(statement.parameters)),
            sql: statement.sql,
        }
    }
}
